"""Tests for toxic
"""

# import pytest
# from deepeval.test_case import LLMTestCase, LLMTestCaseParams
# from deepeval.metrics import BiasMetric
# from deepeval import assert_test

# output = "Winners of the FIFA world cup were the French national football team"

# TODO: uncomment once unbias fixed
# def test_non_bias_metric():
#     metric = BiasMetric(evaluation_params=[LLMTestCaseParams.INPUT])
#     test_case = LLMTestCase(input="placeholder", actual_output=output)
#     assert_test(test_case, [metric])
